@@ -7,12 +7,16 @@
 #include "http_header.h"
 #include "sock_addr.h"
 
+#undef __declspec_dllimport__
+#define __declspec_dllimport__  __declspec_dllexport__
+
 #include "configfile.h"
 #include "plugin.h"
 
 #include <string.h>
 #include <stdlib.h>     /* strtol */
 
+__declspec_dllexport__
 array plugin_stats; /* global */
 
 /**
@@ -496,7 +500,7 @@ static cond_result_t config_check_cond_nocache_eval(request_st * const r, const
 		l = &r->uri.authority;
 		break;
 	case COMP_HTTP_REMOTE_IP:
-		l = &r->con->dst_addr_buf;
+		l = r->dst_addr_buf;
 		break;
 	case COMP_HTTP_SCHEME:
 		l = &r->uri.scheme;
@@ -556,8 +560,8 @@ static cond_result_t config_check_cond_nocache_eval(request_st * const r, const
 			  (((uintptr_t)dc->string.ptr + dc->string.used + 1 + 7) & ~7);
 			int bits = ((unsigned char *)dc->string.ptr)[dc->string.used];
 			match ^= (bits)
-			  ? sock_addr_is_addr_eq_bits(addr, &r->con->dst_addr, bits)
-			  : sock_addr_is_addr_eq(addr, &r->con->dst_addr);
+			  ? sock_addr_is_addr_eq_bits(addr, r->dst_addr, bits)
+			  : sock_addr_is_addr_eq(addr, r->dst_addr);
 			break;
 		}
 		match ^= (buffer_is_equal(l, &dc->string));
